import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
import * as React from "react";
import { Text } from "./Text";
import { CloseButton } from "./IconButton";
import { useTheme } from "./Theme/Providers";
import { useFloating, useClick, useDismiss, useRole, useInteractions, FloatingPortal, FloatingFocusManager, FloatingOverlay, useId } from "@floating-ui/react";
import { mergeRefs } from "react-merge-refs";
export function useDialog({ initialOpen = false, modal = true, open: controlledOpen, onOpenChange: setControlledOpen } = {}) {
    const [uncontrolledOpen, setUncontrolledOpen] = React.useState(initialOpen);
    const [labelId, setLabelId] = React.useState();
    const [descriptionId, setDescriptionId] = React.useState();
    const open = controlledOpen ?? uncontrolledOpen;
    const setOpen = setControlledOpen ?? setUncontrolledOpen;
    const data = useFloating({
        open,
        onOpenChange: setOpen
    });
    const context = data.context;
    const click = useClick(context, {
        enabled: controlledOpen == null
    });
    //确保不会因为点击对话框区域之外立刻关闭地：//modal={state.modal}
    const dismiss = useDismiss(context, {
        outsidePressEvent: "mousedown",
        outsidePress: !modal,
    });
    const role = useRole(context);
    const interactions = useInteractions([click, dismiss, role]);
    return React.useMemo(() => ({
        open,
        setOpen,
        ...interactions,
        ...data,
        labelId,
        descriptionId,
        setLabelId,
        setDescriptionId,
        modal,
    }), [open, setOpen, interactions, data, labelId, descriptionId]);
}
const DialogContext = React.createContext(null);
export const useDialogState = () => {
    const context = React.useContext(DialogContext);
    if (context == null) {
        throw new Error("Dialog components must be wrapped in <Dialog />");
    }
    return context;
};
export function Dialog({ children, ...options }) {
    // This can accept any props as options, e.g. `placement`,
    // or other positioning options.
    const dialog = useDialog(options);
    return (_jsx(DialogContext.Provider, { value: dialog, children: children }));
}
/*自带控制器模式；
可以用<Dialog open={open} onOpenChange={setOpen}>直接代替<DialogTrigger>
* */
export const DialogTrigger = React.forwardRef(function DialogTrigger({ children, asChild = false, ...props }, propRef) {
    const state = useDialogState();
    const childrenRef = children.ref;
    const ref = React.useMemo(() => mergeRefs([state.reference, propRef, childrenRef]), [state.reference, propRef, childrenRef]);
    // `asChild` allows the user to pass any element as the anchor
    if (asChild && React.isValidElement(children)) {
        return React.cloneElement(children, state.getReferenceProps({
            ref,
            ...props,
            ...children.props,
            "data-state": state.open ? "open" : "closed"
        }));
    }
    return (_jsx("button", { ref: ref, "data-state": state.open ? "open" : "closed", ...state.getReferenceProps(props), children: children }));
});
/*对话框主体部分：
全屏幕对话框比如地图那样的，看着和独立有路由器URL赋能的主页面很相似的，就是多了个关闭"X"按钮和标题。
* */
export const DialogContent = React.forwardRef(function DialogContent({ layStyle, ...props }, propRef) {
    const state = useDialogState();
    const ref = React.useMemo(() => mergeRefs([state.floating, propRef]), [
        state.floating,
        propRef
    ]);
    const theme = useTheme();
    //底下的<div "DialogIn" style={{似乎比css={[优先吗，还是因为...props.style注入的更优先的缘故。
    return (_jsx(FloatingPortal, { children: state.open && (_jsx(FloatingOverlay, { className: "Dialog-overlay", lockScroll: true, style: {
                zIndex: theme.zIndices.modal,
                display: 'flex',
                overflow: 'unset',
                overflowX: 'hidden',
                ...layStyle,
            }, children: _jsx(FloatingFocusManager, { context: state.context, modal: state.modal, closeOnFocusOut: false, guards: true, returnFocus: true, children: _jsx("div", { className: "DialogContent", ref: ref, "aria-labelledby": state.labelId, "aria-describedby": state.descriptionId, ...state.getFloatingProps(props), style: {
                        display: 'flex',
                        margin: 'auto',
                        overflowY: 'auto',
                        //这上[theme.mediaQueries.md]: {}的会报错! width放在style={{里导致无法从外面注入到这底下的css={{来覆盖！
                    }, children: _jsx("div", { className: "DialogIn", style: {
                            backgroundColor: 'white',
                            minHeight: "30vh",
                        }, css: {
                            width: "100%",
                            border: '2px solid #ddd',
                            borderRadius: theme.radii.sm,
                            boxShadow: theme.shadows.sm,
                            padding: '4px',
                            [theme.mediaQueries.md]: {
                                margin: '1rem',
                                border: '3px solid #ddd',
                                padding: '8px',
                                maxWidth: "760px",
                                boxShadow: theme.shadows.md,
                                borderRadius: theme.radii.md,
                            },
                            [theme.mediaQueries.lg]: {
                                maxWidth: "990px",
                                boxShadow: theme.shadows["lg"],
                                borderRadius: theme.radii.lg,
                            },
                            "@media print": {
                                boxShadow: 'unset',
                            },
                            ...props.style,
                        }, children: props.children }) }) }) })) }));
});
/*
这底下添加tabIndex={0}才能消除了此毛病：内容超过一个屏幕高度的话，就感觉古怪，直接跳到最后，无法正面显示头部，还要滚动。
* */
export const DialogHeading = React.forwardRef(function DialogHeading({ children, asChild = false, closeStyle, ...props }, ref) {
    const { setLabelId, setOpen } = useDialogState();
    const id = useId();
    // Only sets `aria-labelledby` on the Dialog root element
    // if this component is mounted inside it.
    React.useLayoutEffect(() => {
        setLabelId(id);
        return () => setLabelId(undefined);
    }, [id, setLabelId]);
    //支持更自由的自定义头部：
    const textComp = (asChild && React.isValidElement(children)) ?
        React.cloneElement(children, {
            ...children.props,
        })
        :
            _jsx(Text, { wrap: true, variant: "h5", children: children });
    //似乎只能style={{ 不能改为css={{;
    return (_jsxs("div", { ref: ref, id: id, tabIndex: -1, autoFocus: false, style: {
            justifyContent: 'space-between',
            display: 'flex',
            // ...props.style,
        }, ...props, children: [textComp, _jsx(CloseButton, { style: {
                    ...closeStyle,
                }, onClick: () => setOpen(false) })] }));
});
export const DialogDescription = React.forwardRef(function DialogDescription({ children, ...props }, ref) {
    const { setDescriptionId } = useDialogState();
    const id = useId();
    // Only sets `aria-describedby` on the Dialog root element
    // if this component is mounted inside it.
    React.useLayoutEffect(() => {
        setDescriptionId(id);
        return () => setDescriptionId(undefined);
    }, [id, setDescriptionId]);
    return (_jsx("div", { ...props, ref: ref, id: id, children: children }));
});
export const DialogClose = React.forwardRef(function DialogClose({ children, ...props }, ref) {
    const state = useDialogState();
    return (_jsx("button", { ...props, ref: ref, onClick: () => state.setOpen(false), css: {
            marginTop: '0.5rem',
            marginLeft: '1rem',
        }, children: children }));
});
//# sourceMappingURL=Dialog.js.map